@@ -23,17 +23,76 @@ struct BamHandles
     hts_itr_t* itr;
 };
 
+//
+// SequenceAlignmentRecord
+//
+SequenceAlignmentRecord::SequenceAlignmentRecord(const bam1_t* record)
+{
+    this->read_name = bam_get_qname(record);
+    this->rc = bam_is_rev(record);
+
+    // copy sequence out of the record
+    uint8_t* pseq = bam_get_seq(record);
+    this->sequence.resize(record->core.l_qseq);
+    for(int i = 0; i < record->core.l_qseq; ++i) {
+        this->sequence[i] = seq_nt16_str[bam_seqi(pseq, i)];
+    }
+    
+    // copy read base-to-reference alignment
+    std::vector<AlignedSegment> alignments = get_aligned_segments(record);
+    if(alignments.size() > 1) {
+        fprintf(stderr, "Error: spliced alignments detected when loading read %s\n", this->read_name.c_str());
+        fprintf(stderr, "Please align the reads to the genome using a non-spliced aligner\n");
+        exit(EXIT_FAILURE);
+    }
+    assert(!alignments.empty());
+    this->aligned_bases = alignments[0];
+}
+
+//
+// EventAlignmentRecord
+//
+EventAlignmentRecord::EventAlignmentRecord(SquiggleRead* sr,
+                                           const int strand_idx,
+                                           const SequenceAlignmentRecord& seq_record)
+{
+    this->sr = sr;
+    size_t k = this->sr->get_model_k(strand_idx);
+    size_t read_length = this->sr->read_sequence.length();
+    
+    for(size_t i = 0; i < seq_record.aligned_bases.size(); ++i) {
+        // skip positions at the boundary
+        if(seq_record.aligned_bases[i].read_pos < k) {
+            continue;
+        }
+
+        if(seq_record.aligned_bases[i].read_pos + k >= read_length) {
+            continue;
+        }
+
+        size_t kmer_pos_ref_strand = seq_record.aligned_bases[i].read_pos;
+        size_t kmer_pos_read_strand = seq_record.rc ? this->sr->flip_k_strand(kmer_pos_ref_strand, k) : kmer_pos_ref_strand;
+        size_t event_idx = this->sr->get_closest_event_to(kmer_pos_read_strand, strand_idx);
+        this->aligned_events.push_back( { seq_record.aligned_bases[i].ref_pos, (int)event_idx });
+    }
+    this->rc = strand_idx == 0 ? seq_record.rc : !seq_record.rc;
+    this->strand = strand_idx;
+    this->stride = this->aligned_events.front().read_pos < this->aligned_events.back().read_pos ? 1 : -1;
+}
+
+//
+// AlignmentDB
+//
+
 AlignmentDB::AlignmentDB(const std::string& reads_file,
                          const std::string& reference_file,
                          const std::string& sequence_bam,
-                         const std::string& event_bam,
-                         bool calibrate_reads) :
+                         const std::string& event_bam) :
                             m_reference_file(reference_file),
                             m_sequence_bam(sequence_bam),
-                            m_event_bam(event_bam),
-                            m_fast5_name_map(reads_file),
-                            m_calibrate_on_load(calibrate_reads)
+                            m_event_bam(event_bam)
 {
+    m_read_db.load(reads_file);
     _clear_region();
 }
 
@@ -42,15 +101,29 @@ AlignmentDB::~AlignmentDB()
     _clear_region();
 }
 
+void AlignmentDB::set_alternative_basecalls_bam(const std::string& alternative_basecalls_bam)
+{
+    m_alternative_basecalls_bam = alternative_basecalls_bam;
+}
+
+bool AlignmentDB::are_coordinates_valid(const std::string& contig,
+                                        int start_position,
+                                        int stop_position) const
+{
+    return m_region_contig == contig &&
+           start_position >= m_region_start &&
+           stop_position <= m_region_end;
+}
+
 std::string AlignmentDB::get_reference_substring(const std::string& contig,
                                                  int start_position,
                                                  int stop_position) const
 {
-    assert(m_region_contig == contig);
-    if(m_region_start > start_position || m_region_end < stop_position) {
+    if(!are_coordinates_valid(contig, start_position, stop_position)) {
         fprintf(stderr, "[alignmentdb] error: requested coordinates "
-                "[%d %d] is outside of region boundary [%d %d]\n", 
-                start_position, stop_position, m_region_start, m_region_end);
+                "[%s %d %d] is outside of region boundary [%s %d %d]\n",
+                contig.c_str(), start_position, stop_position,
+                m_region_contig.c_str(), m_region_start, m_region_end);
         exit(EXIT_FAILURE);
     }
 
@@ -107,7 +180,7 @@ std::vector<HMMInputData> AlignmentDB::get_event_subsequences(const std::string&
 
         HMMInputData data;
         data.read = record.sr;
-        data.anchor_index = -1; // unused
+        data.pore_model = record.sr->get_base_model(record.strand);
         data.strand = record.strand;
         data.rc = record.rc;
         data.event_stride = record.stride;
@@ -118,12 +191,19 @@ std::vector<HMMInputData> AlignmentDB::get_event_subsequences(const std::string&
                                            stop_position,
                                            e1,
                                            e2);
+
         if(bounded) {
-            assert(e1 >= 0);
-            assert(e2 >= 0);
-            data.event_start_idx = e1;
-            data.event_stop_idx = e2;
-            out.push_back(data);
+            double ratio = fabs(e1 - e2) / fabs(stop_position - start_position);
+            // Some low quality reads appear to have "stuck" states where you get 
+            // a long run of consecutive stays. They can cause an assertion in the HMM
+            // so we added this heuristic to catch these.
+            if(ratio < MAX_EVENT_TO_BP_RATIO) {
+                assert(e1 >= 0);
+                assert(e2 >= 0);
+                data.event_start_idx = e1;
+                data.event_stop_idx = e2;
+                out.push_back(data);
+            }  
         }
     }
 
@@ -150,7 +230,6 @@ std::vector<HMMInputData> AlignmentDB::get_events_aligned_to(const std::string&
 
         HMMInputData data;
         data.read = record.sr;
-        data.anchor_index = -1; // unused
         data.strand = record.strand;
         data.rc = record.rc;
         data.event_stride = record.stride;
@@ -175,6 +254,9 @@ std::vector<Variant> AlignmentDB::get_variants_in_region(const std::string& cont
 {
     std::vector<Variant> variants;
     std::map<std::string, std::pair<Variant, int>> map;
+    assert(stop_position >= start_position);
+    const int MIN_DISTANCE_TO_REGION_END = 1;
+
     std::vector<int> depth(stop_position - start_position + 1, 0);
 
     for(size_t i = 0; i < m_sequence_records.size(); ++i) {
@@ -234,7 +316,9 @@ std::vector<Variant> AlignmentDB::get_variants_in_region(const std::string& cont
                 }
             }
 
-            if(next_iter != stop_iter && (is_mismatch || is_gap)) {
+            // Make sure this is a variant, that it did not go off the end of the reference and that
+            // it is not too close to the end of the region
+            if(next_iter != stop_iter && (is_mismatch || is_gap) && next_iter->ref_pos < stop_position - MIN_DISTANCE_TO_REGION_END) {
                 Variant v;
                 v.ref_name = contig;
                 v.ref_position = start_iter->ref_pos;
@@ -263,7 +347,7 @@ std::vector<Variant> AlignmentDB::get_variants_in_region(const std::string& cont
         double f = (double)count / d;
         if(f >= min_frequency && (int)d >= min_depth) {
             v.add_info("BaseCalledReadsWithVariant", count);
-            v.add_info("BaseCalledFrequency", f);
+            v.add_info("BaseCalledFraction", f);
             variants.push_back(v);
         }
     }
@@ -276,14 +360,23 @@ void AlignmentDB::load_region(const std::string& contig,
                               int start_position,
                               int stop_position)
 {
-
     // load reference fai file
     faidx_t *fai = fai_load(m_reference_file.c_str());
 
     // Adjust end position to make sure we don't go out-of-range
     m_region_contig = contig;
     m_region_start = start_position;
-    m_region_end = std::min(stop_position, faidx_seq_len(fai, contig.c_str()));
+    int contig_length = faidx_seq_len(fai, contig.c_str());
+    if(contig_length == -1) {
+        fprintf(stderr, "Error: could not retrieve length of contig %s from the faidx.\n", contig.c_str());
+        exit(EXIT_FAILURE);
+    }
+
+    m_region_end = std::min(stop_position, contig_length);
+    
+    assert(!m_region_contig.empty());
+    assert(m_region_start >= 0);
+    assert(m_region_end >= 0);
 
     // load the reference sequence for this region
     // its ok to use the unthreadsafe fetch_seq here since we have our own fai
@@ -292,10 +385,22 @@ void AlignmentDB::load_region(const std::string& contig,
     m_region_ref_sequence = ref_segment;
     
     // load base-space alignments
-    _load_sequence_by_region();
+    m_sequence_records = _load_sequence_by_region(m_sequence_bam);
 
-    // load event-space alignments
-    _load_events_by_region();
+    // load event-space alignments, possibly inferred from the base-space alignments
+    if(m_event_bam.empty()) {
+        m_event_records = _load_events_by_region_from_read(m_sequence_records);
+    } else {
+        m_event_records = _load_events_by_region_from_bam(m_event_bam);
+    }
+
+    // If an alternative basecall set was provided, load it
+    // intentially overwriting the current records
+    if(!m_alternative_basecalls_bam.empty()) {
+        m_sequence_records = _load_sequence_by_region(m_alternative_basecalls_bam);
+    }
+
+    //_debug_print_alignments();
 
     free(ref_segment);
     fai_destroy(fai);
@@ -332,9 +437,10 @@ BamHandles _initialize_bam_itr(const std::string& bam_filename,
     assert(handles.bam_fh != NULL);
 
     // load bam index file
-    std::string index_filename = bam_filename + ".bai";
-    hts_idx_t* bam_idx = bam_index_load(index_filename.c_str());
-    assert(bam_idx != NULL);
+    hts_idx_t* bam_idx = sam_index_load(handles.bam_fh, bam_filename.c_str());
+    if(bam_idx == NULL) {
+        bam_index_error_exit(bam_filename);
+    }
 
     // read the bam header to get the contig ID
     bam_hdr_t* hdr = sam_hdr_read(handles.bam_fh);
@@ -349,51 +455,39 @@ BamHandles _initialize_bam_itr(const std::string& bam_filename,
     return handles;
 }
 
-void AlignmentDB::_load_sequence_by_region()
+std::vector<SequenceAlignmentRecord> AlignmentDB::_load_sequence_by_region(const std::string& sequence_bam)
 {
     assert(!m_region_contig.empty());
     assert(m_region_start >= 0);
     assert(m_region_end >= 0);
 
-    BamHandles handles = _initialize_bam_itr(m_sequence_bam, m_region_contig, m_region_start, m_region_end);
+    BamHandles handles = _initialize_bam_itr(sequence_bam, m_region_contig, m_region_start, m_region_end);
+    std::vector<SequenceAlignmentRecord> records;
 
     int result;
     while((result = sam_itr_next(handles.bam_fh, handles.itr, handles.bam_record)) >= 0) {
-        SequenceAlignmentRecord seq_record;
 
-        // copy sequence out of the record
-        uint8_t* pseq = bam_get_seq(handles.bam_record);
-        seq_record.sequence.resize(handles.bam_record->core.l_qseq);
-        for(int i = 0; i < handles.bam_record->core.l_qseq; ++i) {
-            seq_record.sequence[i] = seq_nt16_str[bam_seqi(pseq, i)];
+        // skip ambiguously mapped reads
+        if(handles.bam_record->core.qual < 20) {
+            continue;
         }
-        
-        // copy read base-to-reference alignment
-        seq_record.aligned_bases = get_aligned_pairs(handles.bam_record);
-        m_sequence_records.push_back(seq_record);
-        
-        /*
-        printf("sequence_record[%zu] prefix: %s alignstart: [%d %d]\n", 
-            m_sequence_records.size() - 1,
-            m_sequence_records.back().sequence.substr(0, 20).c_str(),
-            m_sequence_records.back().aligned_bases.front().ref_pos,
-            m_sequence_records.back().aligned_bases.front().read_pos);
-        */
+
+        records.emplace_back(handles.bam_record);
     }
 
     // cleanup
     sam_itr_destroy(handles.itr);
     bam_destroy1(handles.bam_record);
     sam_close(handles.bam_fh);
+    
+    return records;
 }
 
-void AlignmentDB::_load_events_by_region()
+std::vector<EventAlignmentRecord> AlignmentDB::_load_events_by_region_from_bam(const std::string& event_bam)
 {
-    assert(!m_region_contig.empty());
-    assert(m_region_start >= 0);
-    assert(m_region_end >= 0);
+    BamHandles handles = _initialize_bam_itr(event_bam, m_region_contig, m_region_start, m_region_end);
 
-    BamHandles handles = _initialize_bam_itr(m_event_bam, m_region_contig, m_region_start, m_region_end);
+    std::vector<EventAlignmentRecord> records;
 
     int result;
     while((result = sam_itr_next(handles.bam_fh, handles.itr, handles.bam_record)) >= 0) {
@@ -412,19 +506,7 @@ void AlignmentDB::_load_events_by_region()
         }
 
         std::string read_name = full_name.substr(0, suffix_pos);
-        std::string fast5_path = m_fast5_name_map.get_path(read_name);
-
-        // Do we need to load this fast5 file?
-        if(m_squiggle_read_map.find(read_name) == m_squiggle_read_map.end()) {
-            SquiggleRead* sr = new SquiggleRead(read_name, fast5_path);
-            // Switch the read to use an alternative kmer model
-            if(!m_model_type_string.empty()) {
-                sr->replace_models(m_model_type_string);
-            }
-
-            m_squiggle_read_map[read_name] = sr;
-        }
-
+        _load_squiggle_read(read_name);
         event_record.sr = m_squiggle_read_map[read_name];
 
         // extract the event stride tag which tells us whether the
@@ -433,21 +515,14 @@ void AlignmentDB::_load_events_by_region()
         int event_stride = bam_aux2i(bam_aux_get(handles.bam_record, "ES"));
 
         // copy event alignments
-        event_record.aligned_events = get_aligned_pairs(handles.bam_record, event_stride);
+        std::vector<AlignedSegment> alignments = get_aligned_segments(handles.bam_record, event_stride);
+        assert(alignments.size() > 0);
+        event_record.aligned_events = alignments[0];
 
         event_record.rc = bam_is_rev(handles.bam_record);
         event_record.stride = event_stride;
         event_record.strand = is_template ? T_IDX : C_IDX;
-        m_event_records.push_back(event_record);
-        
-        if(m_calibrate_on_load) {
-            std::vector<EventAlignment> event_alignment = _build_event_alignment(event_record);
-            fprintf(stderr, "Rescale for %s strand: %d rc: %d\n", event_record.sr->read_name.c_str(), event_record.strand, event_record.rc);
-            event_record.sr->print_scaling_parameters(stderr, event_record.strand);
-            fprintf(stderr, "recal events: %zu\n", event_alignment.size());
-            recalibrate_model(*event_record.sr, event_record.strand, event_alignment, &gDNAAlphabet, true, false);
-            event_record.sr->print_scaling_parameters(stderr, event_record.strand);
-        }
+        records.push_back(event_record);
 
         /*
         printf("event_record[%zu] name: %s stride: %d align bounds [%d %d] [%d %d]\n", 
@@ -465,14 +540,85 @@ void AlignmentDB::_load_events_by_region()
     sam_itr_destroy(handles.itr);
     bam_destroy1(handles.bam_record);
     sam_close(handles.bam_fh);
+    
+    return records;
+}
+
+std::vector<EventAlignmentRecord> AlignmentDB::_load_events_by_region_from_read(const std::vector<SequenceAlignmentRecord>& sequence_records)
+{
+    std::vector<EventAlignmentRecord> records;
+    for(size_t i = 0; i < sequence_records.size(); ++i) {
+        const SequenceAlignmentRecord& seq_record = sequence_records[i];
+
+        // conditionally load the squiggle read if it hasn't been loaded already
+        _load_squiggle_read(seq_record.read_name);
+        for(size_t si = 0; si < NUM_STRANDS; ++si) {
+            
+            // skip complement
+            if(si == C_IDX) {
+                continue;
+            }
+    
+            // skip reads that do not have events here
+            SquiggleRead* sr = m_squiggle_read_map[seq_record.read_name];
+            if(!sr->has_events_for_strand(si)) {
+                continue;
+            }
+
+            records.emplace_back(sr, si, seq_record);
+        }
+    }
+
+    return records;
+}
+
+void AlignmentDB::_debug_print_alignments()
+{
+    // Build a map from a squiggle read to the middle base of the reference region it aligned to
+    std::map<std::string, int> read_to_ref_middle;
+    for(size_t i = 0; i < m_sequence_records.size(); ++i) {
+        const SequenceAlignmentRecord& record = m_sequence_records[i];
+        int middle = record.aligned_bases[record.aligned_bases.size()/2].ref_pos;
+        read_to_ref_middle.insert(std::make_pair(record.read_name, middle));
+    }
+
+    for(size_t i = 0; i < m_event_records.size(); ++i) {
+        const EventAlignmentRecord& record = m_event_records[i];
+        AlignedPair first = record.aligned_events.front();
+
+        int ref_middle = read_to_ref_middle[record.sr->read_name];
+        int event_middle_start, event_middle_end;
+        _find_by_ref_bounds(record.aligned_events, ref_middle, ref_middle, event_middle_start, event_middle_end);
+        AlignedPair last = record.aligned_events.back();
+        printf("event_record[%zu] name: %s strand: %d stride: %d rc: %d align bounds [%d %d] [%d %d] [%d %d]\n", 
+                i,
+                record.sr->read_name.c_str(),
+                record.strand,
+                record.stride,
+                record.rc,
+                first.ref_pos,
+                first.read_pos,
+                ref_middle,
+                event_middle_start,
+                last.ref_pos,
+                last.read_pos);
+    }
+}
+
+void AlignmentDB::_load_squiggle_read(const std::string& read_name)
+{
+    // Do we need to load this fast5 file?
+    if(m_squiggle_read_map.find(read_name) == m_squiggle_read_map.end()) {
+        SquiggleRead* sr = new SquiggleRead(read_name, m_read_db);
+        m_squiggle_read_map[read_name] = sr;
+    }
 }
 
 std::vector<EventAlignment> AlignmentDB::_build_event_alignment(const EventAlignmentRecord& event_record) const
 {
     std::vector<EventAlignment> alignment;
     const SquiggleRead* sr = event_record.sr;
-    const Alphabet* alphabet = sr->pore_model[event_record.strand].pmalphabet;
-    size_t k = sr->pore_model[event_record.strand].k;
+    size_t k = sr->get_model_k(event_record.strand);
 
     for(const auto& ap : event_record.aligned_events) { 
 
@@ -505,7 +651,7 @@ bool AlignmentDB::_find_iter_by_ref_bounds(const std::vector<AlignedPair>& pairs
                                       int ref_start,
                                       int ref_stop,
                                       AlignedPairConstIter& start_iter,
-                                      AlignedPairConstIter& stop_iter) const
+                                      AlignedPairConstIter& stop_iter)
 {
     AlignedPairRefLBComp lb_comp;
     start_iter = std::lower_bound(pairs.begin(), pairs.end(),
@@ -532,7 +678,7 @@ bool AlignmentDB::_find_by_ref_bounds(const std::vector<AlignedPair>& pairs,
                                       int ref_start,
                                       int ref_stop,
                                       int& read_start,
-                                      int& read_stop) const
+                                      int& read_stop)
 {
     AlignedPairConstIter start_iter;
     AlignedPairConstIter stop_iter;
