@@ -28,7 +28,7 @@
 #include "nanopolish_matrix.h"
 #include "nanopolish_profile_hmm.h"
 #include "nanopolish_anchor.h"
-#include "nanopolish_fast5_map.h"
+#include "nanopolish_read_db.h"
 #include "nanopolish_hmm_input_sequence.h"
 #include "nanopolish_pore_model_set.h"
 #include "H5pubconf.h"
@@ -63,7 +63,6 @@ static const char *EVENTALIGN_USAGE_MESSAGE =
 "      --progress                       print out a progress message\n"
 "  -n, --print-read-names               print read names instead of indexes\n"
 "      --summary=FILE                   summarize the alignment of each read/strand in FILE\n"
-"      --stdv                           enable stdv modelling\n"
 "      --samples                        write the raw samples for the event to the tsv output\n"
 "      --models-fofn=FILE               read alternative k-mer models from FILE\n"
 "\nReport bugs to " PACKAGE_BUGREPORT "\n\n";
@@ -77,7 +76,6 @@ namespace opt
     static std::string region;
     static std::string summary_file;
     static std::string models_fofn;
-    static std::string alternative_model_type = DEFAULT_MODEL_TYPE;
     static int output_sam = 0;
     static int progress = 0;
     static int num_threads = 1;
@@ -90,7 +88,7 @@ namespace opt
 
 static const char* shortopts = "r:b:g:t:w:vn";
 
-enum { OPT_HELP = 1, OPT_VERSION, OPT_PROGRESS, OPT_SAM, OPT_SUMMARY, OPT_SCALE_EVENTS, OPT_STDV, OPT_MODELS_FOFN, OPT_SAMPLES };
+enum { OPT_HELP = 1, OPT_VERSION, OPT_PROGRESS, OPT_SAM, OPT_SUMMARY, OPT_SCALE_EVENTS, OPT_MODELS_FOFN, OPT_SAMPLES };
 
 static const struct option longopts[] = {
     { "verbose",          no_argument,       NULL, 'v' },
@@ -102,7 +100,6 @@ static const struct option longopts[] = {
     { "summary",          required_argument, NULL, OPT_SUMMARY },
     { "models-fofn",      required_argument, NULL, OPT_MODELS_FOFN },
     { "print-read-names", no_argument,       NULL, 'n' },
-    { "stdv",             no_argument,       NULL, OPT_STDV },
     { "samples",          no_argument,       NULL, OPT_SAMPLES },
     { "scale-events",     no_argument,       NULL, OPT_SCALE_EVENTS },
     { "sam",              no_argument,       NULL, OPT_SAM },
@@ -125,7 +122,7 @@ struct EventalignSummary
 {
     EventalignSummary() {
         num_events = 0;
-        num_matches = 0;
+        num_steps = 0;
         num_stays = 0;
         num_skips = 0;
         sum_z_score = 0;
@@ -135,7 +132,7 @@ struct EventalignSummary
     }
 
     int num_events;
-    int num_matches;
+    int num_steps;
     int num_stays;
     int num_skips;
 
@@ -145,6 +142,16 @@ struct EventalignSummary
     int reference_span;
 };
 
+//
+const PoreModel* EventAlignmentParameters::get_model() const 
+{
+    if(this->alphabet == "") {
+        return this->sr->get_base_model(this->strand_idx);
+    } else {
+        return this->sr->get_model(this->strand_idx, this->alphabet);
+    }
+}
+
 // Modify the aligned_pairs vector to ensure the highest read position
 // does not exceed max_kmer
 void trim_aligned_pairs_to_kmer(std::vector<AlignedPair>& aligned_pairs, int max_kmer_idx)
@@ -371,7 +378,9 @@ void emit_event_alignment_tsv(FILE* fp,
                               const EventAlignmentParameters& params,
                               const std::vector<EventAlignment>& alignments)
 {
-    uint32_t k = sr.pore_model[strand_idx].k;
+    assert(params.alphabet == "");
+    const PoreModel* pore_model = params.get_model();
+    uint32_t k = pore_model->k;
     for(size_t i = 0; i < alignments.size(); ++i) {
 
         const EventAlignment& ea = alignments[i];
@@ -397,21 +406,21 @@ void emit_event_alignment_tsv(FILE* fp,
         }
 
         // event information
-        float event_mean = sr.get_drift_corrected_level(ea.event_idx, ea.strand_idx);
+        float event_mean = sr.get_unscaled_level(ea.event_idx, ea.strand_idx);
         float event_stdv = sr.get_stdv(ea.event_idx, ea.strand_idx);
         float event_duration = sr.get_duration(ea.event_idx, ea.strand_idx);
-        uint32_t rank = params.alphabet->kmer_rank(ea.model_kmer.c_str(), k);
+        uint32_t rank = pore_model->pmalphabet->kmer_rank(ea.model_kmer.c_str(), k);
         float model_mean = 0.0;
         float model_stdv = 0.0;
 
         if(opt::scale_events) {
 
             // scale reads to the model
-            event_mean = (event_mean - sr.pore_model[ea.strand_idx].shift) / sr.pore_model[ea.strand_idx].scale;
+            event_mean = sr.get_fully_scaled_level(ea.event_idx, ea.strand_idx);
 
             // unscaled model parameters
             if(ea.hmm_state != 'B') {
-                PoreModelStateParams model = sr.pore_model[ea.strand_idx].get_parameters(rank);
+                PoreModelStateParams model = pore_model->get_parameters(rank);
                 model_mean = model.level_mean;
                 model_stdv = model.level_stdv;
             }
@@ -419,13 +428,13 @@ void emit_event_alignment_tsv(FILE* fp,
 
             // scale model to the reads
             if(ea.hmm_state != 'B') {
-                GaussianParameters model = sr.pore_model[ea.strand_idx].get_scaled_parameters(rank);
+                GaussianParameters model = sr.get_scaled_gaussian_from_pore_model_state(*pore_model, ea.strand_idx, rank);
                 model_mean = model.mean;
                 model_stdv = model.stdv;
             }
         }
 
-        float standard_level = (event_mean - model_mean) / (sqrt(sr.pore_model[ea.strand_idx].var) * model_stdv);
+        float standard_level = (event_mean - model_mean) / (sqrt(sr.scalings[ea.strand_idx].var) * model_stdv);
         fprintf(fp, "%d\t%.2lf\t%.3lf\t%.5lf\t", ea.event_idx, event_mean, event_stdv, event_duration);
         fprintf(fp, "%s\t%.2lf\t%.2lf\t%.2lf", ea.model_kmer.c_str(),
                                                model_mean,
@@ -453,7 +462,9 @@ EventalignSummary summarize_alignment(const SquiggleRead& sr,
 {
     EventalignSummary summary;
 
-    uint32_t k = sr.pore_model[strand_idx].k;
+    assert(params.alphabet == "");
+    const PoreModel* pore_model = params.get_model();
+    uint32_t k = pore_model->k;
 
     size_t prev_ref_pos = std::string::npos;
 
@@ -469,22 +480,19 @@ EventalignSummary summarize_alignment(const SquiggleRead& sr,
         // movement information
         size_t ref_move = ea.ref_position - prev_ref_pos;
         if(ref_move == 0) {
-            assert(ea.hmm_state == 'E');
             summary.num_stays += 1;
         } else if(i != 0 && ref_move > 1) {
             summary.num_skips += 1;
+        } else if(i != 0 && ref_move == 1) {
+            summary.num_steps += 1;
         }
 
         // event information
         summary.sum_duration += sr.get_duration(ea.event_idx, ea.strand_idx);
 
         if(ea.hmm_state == 'M') {
-            summary.num_matches += 1;
-            
-            uint32_t rank = params.alphabet->kmer_rank(ea.model_kmer.c_str(), k);
-            GaussianParameters model = sr.pore_model[ea.strand_idx].get_scaled_parameters(rank);
-            float event_mean = sr.get_drift_corrected_level(ea.event_idx, ea.strand_idx);
-            double z = (event_mean - model.mean) / model.stdv;
+            uint32_t rank = pore_model->pmalphabet->kmer_rank(ea.model_kmer.c_str(), k);
+            double z = z_score(sr, *pore_model, rank, ea.event_idx, ea.strand_idx);
             summary.sum_z_score += z;
         }
 
@@ -501,7 +509,7 @@ EventalignSummary summarize_alignment(const SquiggleRead& sr,
 
 // Realign the read in event space
 void realign_read(EventalignWriter writer,
-                  const Fast5Map& name_map, 
+                  const ReadDB& read_db, 
                   const faidx_t* fai, 
                   const bam_hdr_t* hdr, 
                   const bam1_t* record, 
@@ -511,14 +519,9 @@ void realign_read(EventalignWriter writer,
 {
     // Load a squiggle read for the mapped read
     std::string read_name = bam_get_qname(record);
-    std::string fast5_path = name_map.get_path(read_name);
 
     // load read
-    SquiggleRead sr(read_name, fast5_path, opt::write_samples ? SRF_LOAD_RAW_SAMPLES : 0);
-
-    if(!opt::alternative_model_type.empty()) {
-        sr.replace_models(opt::alternative_model_type);
-    }
+    SquiggleRead sr(read_name, read_db, opt::write_samples ? SRF_LOAD_RAW_SAMPLES : 0);
 
     if(opt::verbose > 1) {
         fprintf(stderr, "Realigning %s [%zu %zu]\n", 
@@ -560,12 +563,13 @@ void realign_read(EventalignWriter writer,
             }
 
             if(writer.summary_fp != NULL && summary.num_events > 0) {
-
-                PoreModel& pore_model = sr.pore_model[strand_idx];
+                assert(params.alphabet == "");
+                const PoreModel* pore_model = params.get_model();
+                SquiggleScalings& scalings = sr.scalings[strand_idx];
                 fprintf(writer.summary_fp, "%zu\t%s\t%s\t", read_idx, read_name.c_str(), sr.fast5_path.c_str());
-                fprintf(writer.summary_fp, "%s\t%s\t", pore_model.name.c_str(), strand_idx == 0 ? "template" : "complement");
-                fprintf(writer.summary_fp, "%d\t%d\t%d\t%d\t", summary.num_events, summary.num_matches, summary.num_skips, summary.num_stays);
-                fprintf(writer.summary_fp, "%.2lf\t%.3lf\t%.3lf\t%.3lf\t%.3lf\n", summary.sum_duration, pore_model.shift, pore_model.scale, pore_model.drift, pore_model.var);
+                fprintf(writer.summary_fp, "%s\t%s\t", pore_model->name.c_str(), strand_idx == 0 ? "template" : "complement");
+                fprintf(writer.summary_fp, "%d\t%d\t%d\t%d\t", summary.num_events, summary.num_steps, summary.num_skips, summary.num_stays);
+                fprintf(writer.summary_fp, "%.2lf\t%.3lf\t%.3lf\t%.3lf\t%.3lf\n", summary.sum_duration, scalings.shift, scalings.scale, scalings.drift, scalings.var);
             }
         }
     }
@@ -580,6 +584,7 @@ std::vector<EventAlignment> align_read_to_ref(const EventAlignmentParameters& pa
     assert(params.record != NULL);
     assert(params.strand_idx < NUM_STRANDS);
     assert( (params.region_start == -1 && params.region_end == -1) || (params.region_start <= params.region_end));
+    const PoreModel* pore_model = params.get_model();
 
     std::vector<EventAlignment> alignment_output;
 
@@ -591,186 +596,192 @@ std::vector<EventAlignment> align_read_to_ref(const EventAlignmentParameters& pa
                                                   bam_endpos(params.record), &fetched_len);
 
     // k from read pore model
-    const uint32_t k = params.sr->pore_model[params.strand_idx].k;
+    const uint32_t k = params.sr->get_model_k(params.strand_idx);
 
     // If the reference sequence contains ambiguity codes
     // switch them to the lexicographically lowest base
-    ref_seq = params.alphabet->disambiguate(ref_seq);
-    std::string rc_ref_seq = params.alphabet->reverse_complement(ref_seq);
+    ref_seq = pore_model->pmalphabet->disambiguate(ref_seq);
+    std::string rc_ref_seq = pore_model->pmalphabet->reverse_complement(ref_seq);
 
     if(ref_offset == 0)
         return alignment_output;
 
-    // Make a vector of aligned (ref_pos, read_pos) pairs
-    std::vector<AlignedPair> aligned_pairs = get_aligned_pairs(params.record);
+    // Get the read-to-reference aligned segments
+    std::vector<AlignedSegment> aligned_segments = get_aligned_segments(params.record);
+    for(size_t segment_idx = 0; segment_idx < aligned_segments.size(); ++segment_idx) {
 
-    if(params.region_start != -1 && params.region_end != -1) {
-        trim_aligned_pairs_to_ref_region(aligned_pairs, params.region_start, params.region_end);
-    }
+        AlignedSegment& aligned_pairs = aligned_segments[segment_idx];
 
-    // Trim the aligned pairs to be within the range of the maximum kmer index
-    int max_kmer_idx = params.sr->read_sequence.size() - k;
-    trim_aligned_pairs_to_kmer(aligned_pairs, max_kmer_idx);
+        if(params.region_start != -1 && params.region_end != -1) {
+            trim_aligned_pairs_to_ref_region(aligned_pairs, params.region_start, params.region_end);
+        }
 
-    if(aligned_pairs.empty())
-        return alignment_output;
+        // Trim the aligned pairs to be within the range of the maximum kmer index
+        int max_kmer_idx = params.sr->read_sequence.size() - k;
+        trim_aligned_pairs_to_kmer(aligned_pairs, max_kmer_idx);
 
-    bool do_base_rc = bam_is_rev(params.record);
-    bool rc_flags[2] = { do_base_rc, !do_base_rc }; // indexed by strand
-    const int align_stride = 100; // approximately how many reference bases to align to at once
-    const int output_stride = 50; // approximately how many event alignments to output at once
+        if(aligned_pairs.empty())
+            return alignment_output;
 
-    // get the event range of the read to re-align
-    int read_kidx_start = aligned_pairs.front().read_pos;
-    int read_kidx_end = aligned_pairs.back().read_pos;
-    
-    if(do_base_rc) {
-        read_kidx_start = params.sr->flip_k_strand(read_kidx_start);
-        read_kidx_end = params.sr->flip_k_strand(read_kidx_end);
-    }
-    
-    assert(read_kidx_start >= 0);
-    assert(read_kidx_end >= 0);
-
-    int first_event = params.sr->get_closest_event_to(read_kidx_start, params.strand_idx);
-    int last_event = params.sr->get_closest_event_to(read_kidx_end, params.strand_idx);
-    bool forward = first_event < last_event;
-
-    int curr_start_event = first_event;
-    int curr_start_ref = aligned_pairs.front().ref_pos;
-    int curr_pair_idx = 0;
-
-    while( (forward && curr_start_event < last_event) ||
-           (!forward && curr_start_event > last_event)) {
-
-        // Get the index of the aligned pair approximately align_stride away
-        int end_pair_idx = get_end_pair(aligned_pairs, curr_start_ref + align_stride, curr_pair_idx);
-    
-        int curr_end_ref = aligned_pairs[end_pair_idx].ref_pos;
-        int curr_end_read = aligned_pairs[end_pair_idx].read_pos;
+        bool do_base_rc = bam_is_rev(params.record);
+        bool rc_flags[2] = { do_base_rc, !do_base_rc }; // indexed by strand
+        const int align_stride = 100; // approximately how many reference bases to align to at once
+        const int output_stride = 50; // approximately how many event alignments to output at once
 
+        // get the event range of the read to re-align
+        int read_kidx_start = aligned_pairs.front().read_pos;
+        int read_kidx_end = aligned_pairs.back().read_pos;
+        
         if(do_base_rc) {
-            curr_end_read = params.sr->flip_k_strand(curr_end_read);
+            read_kidx_start = params.sr->flip_k_strand(read_kidx_start, k);
+            read_kidx_end = params.sr->flip_k_strand(read_kidx_end, k);
         }
-        assert(curr_end_read >= 0);
-
-        int s = curr_start_ref - ref_offset;
-        int l = curr_end_ref - curr_start_ref + 1;
-
-        std::string fwd_subseq = ref_seq.substr(s, l);
-        std::string rc_subseq = rc_ref_seq.substr(ref_seq.length() - s - l, l);
-        assert(fwd_subseq.length() == rc_subseq.length());
-
-        HMMInputSequence hmm_sequence(fwd_subseq, rc_subseq, params.alphabet);
         
-        // Require a minimum amount of sequence to align to
-        if(hmm_sequence.length() < 2 * k)
-            break;
+        assert(read_kidx_start >= 0);
+        assert(read_kidx_end >= 0);
 
-        // Set up HMM input
-        HMMInputData input;
-        input.read = params.sr;
-        input.anchor_index = 0; // not used here
-        input.event_start_idx = curr_start_event;
-        input.event_stop_idx = params.sr->get_closest_event_to(curr_end_read, params.strand_idx);
-        //printf("[SEGMENT_START] read: %s event start: %zu event end: %zu\n", params.sr->read_name.c_str(), input.event_start_idx, input.event_stop_idx);
+        int first_event = params.sr->get_closest_event_to(read_kidx_start, params.strand_idx);
+        int last_event = params.sr->get_closest_event_to(read_kidx_end, params.strand_idx);
+        bool forward = first_event < last_event;
 
-        // A limitation of the segment-by-segment alignment is that we can't jump
-        // over very large deletions wrt to the reference. The effect of this
-        // is that we can get segments that have very few alignable events. We
-        // just stop processing them for now
-        if(abs(input.event_start_idx - input.event_stop_idx) < 2)
-            break;
+        int curr_start_event = first_event;
+        int curr_start_ref = aligned_pairs.front().ref_pos;
+        int curr_pair_idx = 0;
 
-        input.strand = params.strand_idx;
-        input.event_stride = input.event_start_idx < input.event_stop_idx ? 1 : -1;
-        input.rc = rc_flags[params.strand_idx];
+        while( (forward && curr_start_event < last_event) ||
+               (!forward && curr_start_event > last_event)) {
 
-        std::vector<HMMAlignmentState> event_alignment = profile_hmm_align(hmm_sequence, input);
+            // Get the index of the aligned pair approximately align_stride away
+            int end_pair_idx = get_end_pair(aligned_pairs, curr_start_ref + align_stride, curr_pair_idx);
         
-        // Output alignment
-        size_t num_output = 0;
-        size_t event_align_idx = 0;
+            int curr_end_ref = aligned_pairs[end_pair_idx].ref_pos;
+            int curr_end_read = aligned_pairs[end_pair_idx].read_pos;
 
-        // If we aligned to the last event, output everything and stop
-        bool last_section = end_pair_idx == (int)aligned_pairs.size() - 1;
-
-        /*
-        // Don't allow the segment to end on an E state or else we get alignment
-        // artifacts at the segment boundary
-        if(!last_section) {
-            size_t last_match_index = event_alignment.size() - 1;
-            while(event_alignment[last_match_index].state != 'M') {
-                last_match_index -= 1;
+            if(do_base_rc) {
+                curr_end_read = params.sr->flip_k_strand(curr_end_read, k);
             }
+            assert(curr_end_read >= 0);
 
-            event_alignment.resize(last_match_index + 1);
-            if(event_alignment.empty()) {
+            int s = curr_start_ref - ref_offset;
+            int l = curr_end_ref - curr_start_ref + 1;
+
+            std::string fwd_subseq = ref_seq.substr(s, l);
+            std::string rc_subseq = rc_ref_seq.substr(ref_seq.length() - s - l, l);
+            assert(fwd_subseq.length() == rc_subseq.length());
+
+            HMMInputSequence hmm_sequence(fwd_subseq, rc_subseq, pore_model->pmalphabet);
+            
+            // Require a minimum amount of sequence to align to
+            if(hmm_sequence.length() < 2 * k)
                 break;
-            }
-            assert(event_alignment.back().state == 'M');
-        }
-        */
 
-        int last_event_output = 0;
-        int last_ref_kmer_output = 0;
+            // Set up HMM input
+            HMMInputData input;
+            input.read = params.sr;
+            input.pore_model = pore_model;
+            assert(input.pore_model != NULL);
 
-        for(; event_align_idx < event_alignment.size() && 
-              (num_output < output_stride || last_section); event_align_idx++) {
+            input.event_start_idx = curr_start_event;
+            input.event_stop_idx = params.sr->get_closest_event_to(curr_end_read, params.strand_idx);
+            //printf("[SEGMENT_START] read: %s event start: %zu event end: %zu\n", params.sr->read_name.c_str(), input.event_start_idx, input.event_stop_idx);
 
-            HMMAlignmentState& as = event_alignment[event_align_idx];
-            if(as.state != 'K' && (int)as.event_idx != curr_start_event) {
+            // A limitation of the segment-by-segment alignment is that we can't jump
+            // over very large deletions wrt to the reference. The effect of this
+            // is that we can get segments that have very few alignable events. We
+            // just stop processing them for now
+            if(abs((int)input.event_start_idx - (int)input.event_stop_idx) < 2)
+                break;
 
-                EventAlignment ea;
-                
-                // ref
-                ea.ref_name = ref_name;
-                ea.ref_position = curr_start_ref + as.kmer_idx;
-                ea.ref_kmer = ref_seq.substr(ea.ref_position - ref_offset, k);
+            input.strand = params.strand_idx;
+            input.event_stride = input.event_start_idx < input.event_stop_idx ? 1 : -1;
+            input.rc = rc_flags[params.strand_idx];
 
-                // event
-                ea.read_idx = params.read_idx;
-                ea.strand_idx = params.strand_idx;
-                ea.event_idx = as.event_idx;
-                ea.rc = input.rc;
+            std::vector<HMMAlignmentState> event_alignment = profile_hmm_align(hmm_sequence, input);
+            
+            // Output alignment
+            size_t num_output = 0;
+            size_t event_align_idx = 0;
 
-                // hmm
-                ea.hmm_state = as.state;
+            // If we aligned to the last event, output everything and stop
+            bool last_section = end_pair_idx == (int)aligned_pairs.size() - 1;
 
-                if(ea.hmm_state != 'B') {
-                    ea.model_kmer = hmm_sequence.get_kmer(as.kmer_idx, k, input.rc);
-                } else {
-                    ea.model_kmer = std::string(k, 'N');
+            /*
+            // Don't allow the segment to end on an E state or else we get alignment
+            // artifacts at the segment boundary
+            if(!last_section) {
+                size_t last_match_index = event_alignment.size() - 1;
+                while(event_alignment[last_match_index].state != 'M') {
+                    last_match_index -= 1;
                 }
 
-                // store
-                alignment_output.push_back(ea);
-
-                // update
-                last_event_output = as.event_idx;
-                last_ref_kmer_output = curr_start_ref + as.kmer_idx;
-
-                num_output += 1;
+                event_alignment.resize(last_match_index + 1);
+                if(event_alignment.empty()) {
+                    break;
+                }
+                assert(event_alignment.back().state == 'M');
             }
-        }
+            */
 
-        // Advance the pair iterator to the ref base
-        curr_start_event = last_event_output;
-        curr_start_ref = last_ref_kmer_output;
-        //printf("[SEGMENT_END] read: %s last event output: %zu ref pos: %zu (%s)\n", params.sr->read_name.c_str(), last_event_output, last_ref_kmer_output, ref_seq.substr(last_ref_kmer_output - ref_offset, k).c_str());
-        curr_pair_idx = get_end_pair(aligned_pairs, curr_start_ref, curr_pair_idx);
+            int last_event_output = 0;
+            int last_ref_kmer_output = 0;
+
+            for(; event_align_idx < event_alignment.size() && 
+                  (num_output < output_stride || last_section); event_align_idx++) {
+
+                HMMAlignmentState& as = event_alignment[event_align_idx];
+                if(as.state != 'K' && (int)as.event_idx != curr_start_event) {
+
+                    EventAlignment ea;
+                    
+                    // ref
+                    ea.ref_name = ref_name;
+                    ea.ref_position = curr_start_ref + as.kmer_idx;
+                    ea.ref_kmer = ref_seq.substr(ea.ref_position - ref_offset, k);
+
+                    // event
+                    ea.read_idx = params.read_idx;
+                    ea.strand_idx = params.strand_idx;
+                    ea.event_idx = as.event_idx;
+                    ea.rc = input.rc;
+
+                    // hmm
+                    ea.hmm_state = as.state;
+
+                    if(ea.hmm_state != 'B') {
+                        ea.model_kmer = hmm_sequence.get_kmer(as.kmer_idx, k, input.rc);
+                    } else {
+                        ea.model_kmer = std::string(k, 'N');
+                    }
+
+                    // store
+                    alignment_output.push_back(ea);
+
+                    // update
+                    last_event_output = as.event_idx;
+                    last_ref_kmer_output = curr_start_ref + as.kmer_idx;
+
+                    num_output += 1;
+                }
+            }
+
+            // Advance the pair iterator to the ref base
+            curr_start_event = last_event_output;
+            curr_start_ref = last_ref_kmer_output;
+            //printf("[SEGMENT_END] read: %s last event output: %zu ref pos: %zu (%s)\n", params.sr->read_name.c_str(), last_event_output, last_ref_kmer_output, ref_seq.substr(last_ref_kmer_output - ref_offset, k).c_str());
+            curr_pair_idx = get_end_pair(aligned_pairs, curr_start_ref, curr_pair_idx);
 
 #if EVENTALIGN_TRAIN
-        // update training data for read
-        params.sr->parameters[params.strand_idx].add_training_from_alignment(hmm_sequence, input, event_alignment);
-        global_training[params.strand_idx].add_training_from_alignment(hmm_sequence, input, event_alignment);
+            // update training data for read
+            params.sr->parameters[params.strand_idx].add_training_from_alignment(hmm_sequence, input, event_alignment);
+            global_training[params.strand_idx].add_training_from_alignment(hmm_sequence, input, event_alignment);
 #endif
 
-        if(num_output == 0) {
-            break;
-        }
-    } // for segment
-    
+            if(num_output == 0) {
+                break;
+            }
+        } // for realignmentsegment
+    } // for bam aligned segment
+
     return alignment_output;
 }
 
@@ -787,7 +798,6 @@ void parse_eventalign_options(int argc, char** argv)
             case 't': arg >> opt::num_threads; break;
             case 'n': opt::print_read_names = true; break;
             case 'f': opt::full_output = true; break;
-            case OPT_STDV: model_stdv() = true; break;
             case OPT_SAMPLES: opt::write_samples = true; break;
             case 'v': opt::verbose++; break;
             case OPT_MODELS_FOFN: arg >> opt::models_fofn; break;
@@ -833,10 +843,7 @@ void parse_eventalign_options(int argc, char** argv)
         die = true;
     }
 
-    if(opt::models_fofn.empty()) {
-        std::cerr << SUBPROGRAM ": a --models file must be provided\n";
-        die = true;
-    } else {
+    if(!opt::models_fofn.empty()) {
         // initialize the model set from the fofn
         PoreModelSet::initialize(opt::models_fofn);
     }
@@ -853,7 +860,8 @@ int eventalign_main(int argc, char** argv)
     parse_eventalign_options(argc, argv);
     omp_set_num_threads(opt::num_threads);
 
-    Fast5Map name_map(opt::reads_file);
+    ReadDB read_db;
+    read_db.load(opt::reads_file);
     
     // Open the BAM and iterate over reads
 
@@ -864,7 +872,9 @@ int eventalign_main(int argc, char** argv)
     // load bam index file
     std::string index_filename = opt::bam_file + ".bai";
     hts_idx_t* bam_idx = bam_index_load(index_filename.c_str());
-    assert(bam_idx != NULL);
+    if(bam_idx == NULL) {
+        bam_index_error_exit(opt::bam_file);
+    }
 
     // read the bam header
     bam_hdr_t* hdr = sam_hdr_read(bam_fh);
@@ -911,7 +921,7 @@ int eventalign_main(int argc, char** argv)
         writer.summary_fp = fopen(opt::summary_file.c_str(), "w");
         // header
         fprintf(writer.summary_fp, "read_index\tread_name\tfast5_path\tmodel_name\tstrand\tnum_events\t");
-        fprintf(writer.summary_fp, "num_matches\tnum_skips\tnum_stays\ttotal_duration\tshift\tscale\tdrift\tvar\n");
+        fprintf(writer.summary_fp, "num_steps\tnum_skips\tnum_stays\ttotal_duration\tshift\tscale\tdrift\tvar\n");
     }
     
     // Initialize iteration
@@ -930,8 +940,8 @@ int eventalign_main(int argc, char** argv)
         
         // read a record into the next slot in the buffer
         result = sam_itr_next(bam_fh, itr, records[num_records_buffered]);
+        
         num_records_buffered += result >= 0;
-
         // realign if we've hit the max buffer size or reached the end of file
         if(num_records_buffered == records.size() || result < 0) {
             #pragma omp parallel for            
@@ -939,7 +949,7 @@ int eventalign_main(int argc, char** argv)
                 bam1_t* record = records[i];
                 size_t read_idx = num_reads_realigned + i;
                 if( (record->core.flag & BAM_FUNMAP) == 0) {
-                    realign_read(writer, name_map, fai, hdr, record, read_idx, clip_start, clip_end);
+                    realign_read(writer, read_db, fai, hdr, record, read_idx, clip_start, clip_end);
                 }
             }
 
