@@ -18,7 +18,7 @@
 #include "logsum.h"
 
 #define PACKAGE_NAME "nanopolish"
-#define PACKAGE_VERSION "0.5.0"
+#define PACKAGE_VERSION "0.8.4"
 #define PACKAGE_BUGREPORT "https://github.com/jts/nanopolish/issues"
 
 //
@@ -43,15 +43,17 @@ const uint8_t NUM_STRANDS = 2;
 // Data structures
 //
 
+// Forward declare
 class SquiggleRead;
+class PoreModel;
 
 // This struct is used as input into the HMM
-// It tracks where the event stream starts/stops
-// for the partial consensus sequence under consideration
+// It tracks where the event stream starts/stops for the candidate sequence
+// that is being evaluated.
 struct HMMInputData
 {
     SquiggleRead* read;
-    uint32_t anchor_index;
+    const PoreModel* pore_model;
     uint32_t event_start_idx;
     uint32_t event_stop_idx;
     uint8_t strand;
@@ -73,7 +75,7 @@ struct HMMAlignmentState
 // The parameters of a gaussian distribution
 struct GaussianParameters
 {
-    GaussianParameters() : mean(0.0f), stdv(1.0f) { log_stdv = log(stdv); }
+    GaussianParameters() : mean(0.0f), stdv(1.0f), log_stdv(0.0) { }
     GaussianParameters(float m, float s) : mean(m), stdv(s) { log_stdv = log(stdv); }
 
     float mean;
@@ -81,6 +83,14 @@ struct GaussianParameters
     float log_stdv; // == log(stdv), pre-computed for efficiency
 };
 
+//
+struct SemVer
+{
+    int major;
+    int minor;
+    int patch;
+};
+
 //
 // Functions
 //
@@ -111,6 +121,12 @@ std::vector<std::string> split(std::string in, char delimiter);
 // returns true if the provided string ends with the given suffix
 bool ends_with(const std::string& str, const std::string& suffix);
 
+// parse a region string (chr:start-end)
+void parse_region_string(const std::string& region, std::string& contig, int& start, int& end);
+
+// parse a software version string using the semver convention
+SemVer parse_semver_string(const std::string& semver_str);
+
 // from: http://stackoverflow.com/questions/9330915/number-of-combinations-n-choose-r-in-c
 size_t nChoosek(size_t n, size_t k);
 
@@ -122,6 +138,15 @@ size_t nChoosek(size_t n, size_t k);
 template<class T>
 std::string array2str(const T& array)
 {
-    return std::string(array.begin(), array.end());
+    std::string s(array.data(), array.size());
+    size_t null_pos = s.find('\0');
+    if(null_pos != std::string::npos) {
+        s.erase(null_pos);
+    }
+    return s;
 }
+
+// Print an error message when failing to load a bam index and exit
+void bam_index_error_exit(const std::string& bam_filename);
+
 #endif
