@@ -9,28 +9,33 @@
 #include <map>
 #include <functional>
 #include "logsum.h"
+#include "nanopolish_index.h"
+#include "nanopolish_extract.h"
 #include "nanopolish_call_variants.h"
-#include "nanopolish_consensus.h"
 #include "nanopolish_eventalign.h"
 #include "nanopolish_getmodel.h"
 #include "nanopolish_methyltrain.h"
-#include "nanopolish_methyltest.h"
+#include "nanopolish_call_methylation.h"
 #include "nanopolish_scorereads.h"
+#include "nanopolish_phase_reads.h"
 #include "nanopolish_train_poremodel_from_basecalls.h"
 
 int print_usage(int argc, char **argv);
+int print_version(int argc, char **argv);
 
 static std::map< std::string, std::function<int(int, char**)> > programs = {
     {"help",        print_usage},
     {"--help",      print_usage},
-    {"consensus",   consensus_main},
+    {"--version",   print_version},
+    {"index",       index_main},
+    {"extract",     extract_main},
     {"eventalign",  eventalign_main},
     {"getmodel",    getmodel_main},
     {"variants",    call_variants_main},
     {"methyltrain", methyltrain_main},
-    {"methyltest",  methyltest_main},
     {"scorereads",  scorereads_main} ,
-    {"train-poremodel-from-basecalls",  train_poremodel_from_basecalls_main}
+    {"phase-reads",  phase_reads_main} ,
+    {"call-methylation",  call_methylation_main}
 };
 
 int print_usage(int, char **)
@@ -39,13 +44,25 @@ int print_usage(int, char **)
     std::cout << "  valid commands: " << std::endl;
     for (const auto &item : programs){
         std::cout << "    " << item.first << std::endl;
-    } 
+    }
     std::cout << "  for help on given command, type nanopolish command --help" << std::endl;
     return 0;
 }
 
+int print_version(int, char **)
+{
+    static const char *VERSION_MESSAGE =
+    "nanopolish version " PACKAGE_VERSION "\n"
+    "Written by Jared Simpson.\n"
+    "\n"
+    "Copyright 2015-2017 Ontario Institute for Cancer Research\n";
+    std::cout << VERSION_MESSAGE << std::endl;
+    return 0;
+}
+
 int main(int argc, char** argv)
 {
+    int ret = 0;
     if(argc <= 1) {
         printf("error: no command provided\n");
         print_usage(argc - 1 , argv + 1);
@@ -54,8 +71,20 @@ int main(int argc, char** argv)
         std::string command(argv[1]);
         auto iter = programs.find(command);
         if (iter != programs.end()) 
-            return iter->second( argc - 1, argv + 1);
+            ret = iter->second( argc - 1, argv + 1);
         else
-            return print_usage( argc - 1, argv + 1);
+            ret = print_usage( argc - 1, argv + 1);
     }
+
+    // Emit a warning when some reads had to be skipped
+    extern int g_total_reads;
+    extern int g_unparseable_reads;
+    extern int g_qc_fail_reads;
+    extern int g_failed_calibration_reads;
+    extern int g_failed_alignment_reads;
+    if(g_total_reads > 0) {
+        fprintf(stderr, "[post-run summary] total reads: %d unparseable: %d qc fail: %d could not calibrate: %d no alignment: %d\n", 
+            g_total_reads, g_unparseable_reads, g_qc_fail_reads, g_failed_calibration_reads, g_failed_alignment_reads);
+    }
+    return ret;
 }
